@@ -140,8 +140,6 @@ err_t pack_whack_msg (struct whackpacker *wp)
     	|| !pack_str(wp, &wp->msg->left.xauth_name)    /* string 19 */
     	|| !pack_str(wp, &wp->msg->right.xauth_name)   /* string 20 */
     	|| !pack_str(wp, &wp->msg->connalias)   /* string 21 */
-    	|| !pack_str(wp, &wp->msg->left.host_addr_name)    /* string 22 */
-    	|| !pack_str(wp, &wp->msg->right.host_addr_name)   /* string 23 */
 	|| wp->str_roof - wp->str_next < (ptrdiff_t)wp->msg->keyval.len)    /* chunk (sort of string 19) */
     {
 	ugh = "too many bytes of strings to fit in message to pluto";
@@ -195,8 +193,6 @@ err_t unpack_whack_msg (struct whackpacker *wp)
     	|| !unpack_str(wp, &wp->msg->left.xauth_name)    /* string 19 */
     	|| !unpack_str(wp, &wp->msg->right.xauth_name)   /* string 20 */
     	|| !unpack_str(wp, &wp->msg->connalias)   /* string 21 */
-    	|| !unpack_str(wp, &wp->msg->left.host_addr_name)    /* string 22 */
-    	|| !unpack_str(wp, &wp->msg->right.host_addr_name)   /* string 23 */
 	|| wp->str_roof - wp->str_next != (ptrdiff_t)wp->msg->keyval.len)	/* check chunk */
     {
 	ugh = "message from whack contains bad string";
