@@ -39,15 +39,23 @@ __FBSDID("$FreeBSD: src/sys/opencrypto/crypto.c,v 1.16 2005/01/07 02:29:16 imp E
 #include <linux/sched.h>
 #include <linux/spinlock.h>
 #include <linux/version.h>
+#include <linux/debugfs.h>
+#include <linux/debugfs_circular.h>
+#include <linux/jiffies.h>
+#include <asm/atomic.h>
 #include <opencrypto/crypto.h>
 #include <opencrypto/cryptodev.h>
+#include <opencrypto/cryptoprof.h>
+#include <stdbool.h>
 
 /*
- * keep track of whether or not we have been initialised, a big
- * issue if we are linked into the kernel and a driver gets started before
- * us
+ * keep track of whether or not we have been initialised, a big issue 
+ * if we are linked into the kernel and a driver gets called before us.
+ * 
+ * The value starts off at -1 and is incremented in crypto_init().  Any
+ * non-negative value indicates initialization was done already.
  */
-static int crypto_initted = 0;
+static atomic_t crypto_needs_init = ATOMIC_INIT(-1);
 
 /*
  * Crypto drivers register themselves by allocating a slot in the
@@ -79,7 +87,10 @@ static int crypto_drivers_num = 0;
 static LIST_HEAD(crp_q);		/* request queues */
 static LIST_HEAD(crp_kq);
 
-static int crypto_q_locked = 0;	/* on !SMP systems, spin locks do nothing :-( */
+static int crypto_q_locked = 0;	/* on !SMP systems, spin locks just disable
+				 * interrupts, which keeps us from noticing
+				 * things for debugging. */
+
 static spinlock_t crypto_q_lock;
 #define	CRYPTO_Q_LOCK() \
 			({ \
@@ -125,6 +136,15 @@ module_param(debug, int, 0644);
 MODULE_PARM_DESC(debug,
 	   "Enable debug");
 
+static int crypto_proc_debug = 0;
+module_param(crypto_proc_debug, int, 0644);
+MODULE_PARM_DESC(crypto_proc_debug,
+		 "Enable debug of thread");
+
+struct debugfs_circular *crypto_proc_circ;
+
+#define cp_printk(fmt,msg...) if(crypto_proc_debug) printk(fmt,##msg)
+
 /*
  * Maximum number of outstanding crypto requests before we start
  * failing requests.  We need this to prevent DOS when too many
@@ -136,7 +156,21 @@ MODULE_PARM_DESC(debug,
  * slow,  printing anything will just kill us
  */
 
+/* we never let atomic's get set */
+int param_set_atomic_t(const char *val, struct kernel_param *kp)
+{
+	return 0;
+}
+
+int param_get_atomic_t(char *buffer, struct kernel_param *kp)
+{
+	return sprintf(buffer, "%u", atomic_read((atomic_t *)kp->arg));
+}
+#define param_check_atomic_t(name, p) __param_check(name, p, atomic_t)
+
 static atomic_t crypto_q_cnt;
+module_param(crypto_q_cnt, atomic_t, 0444);
+
 static int crypto_q_max = 1000;
 module_param(crypto_q_max, int, 0644);
 MODULE_PARM_DESC(crypto_q_max,
@@ -163,13 +197,79 @@ module_param(crypto_devallowsoft, int, 0644);
 MODULE_PARM_DESC(crypto_devallowsoft,
 	   "Enable/disable use of software asym crypto support");
 
+static int	crypto_qsblocked = 0;	/* symmetric requests on blocked q */
+static int	crypto_qs = 0;	        /* number of symmetric requests */
+static int	crypto_qkblocked = 0;	/* number of asymmetric requests */
+module_param(crypto_qsblocked, int, 0444);
+module_param(crypto_qs, int, 0444);
+module_param(crypto_qkblocked, int, 0444);
+
 static pid_t	cryptoproc = (pid_t) -1;
 static struct	completion cryptoproc_exited;
 static DECLARE_WAIT_QUEUE_HEAD(cryptoproc_wait);
+
+uint      cryptoproc_active = 0;
+module_param(cryptoproc_active, uint, 0644);
+MODULE_PARM_DESC(cryptoproc_active,
+		  "indicates instantenous number of requests that need the"
+		  "submission thread to be active.");
+
+uint cryptoproc_falsewakes=0;
+module_param(cryptoproc_falsewakes, uint, 0444);
+
+MODULE_PARM_DESC(cryptoproc_lastditch,
+		 "counts number of times, a second attempt at the list"
+		 "showed something that actually needed processing"); 
+
+uint      cryptoproc_trylastditch = 0;
+module_param(cryptoproc_trylastditch, uint, 0444);
+
+uint      cryptoproc_lastditch = 0;
+module_param(cryptoproc_lastditch, uint, 0444);
+MODULE_PARM_DESC(cryptoproc_lastditch,
+		 "counts number of times, a second attempt at the list"
+		 "showed something that actually needed processing"); 
+
 static pid_t	cryptoretproc = (pid_t) -1;
 static struct	completion cryptoretproc_exited;
 static DECLARE_WAIT_QUEUE_HEAD(cryptoretproc_wait);
 
+int      cryptoproc_unblock_called=0;
+module_param(cryptoproc_unblock_called, int, 0444);
+MODULE_PARM_DESC(cryptoproc_unblock_called,
+		 "number of times the crypto_proc() thread was woken up");
+
+int      cryptoproc_unblock_qnotblocked=0;
+module_param(cryptoproc_unblock_qnotblocked, int, 0444);
+MODULE_PARM_DESC(cryptoproc_unblock_qnotblocked,
+		 "number of times the crypto_proc() thread was woken up");
+
+int      cryptoproc_unblock=0;
+module_param(cryptoproc_unblock, int, 0444);
+MODULE_PARM_DESC(cryptoproc_unblock,
+		 "number of times the crypto_proc() thread was woken up");
+
+int      cryptoproc_unblock_unneeded=0;
+module_param(cryptoproc_unblock_unneeded, int, 0444);
+MODULE_PARM_DESC(cryptoproc_unblock_unneeded,
+		 "number of times the crypto_proc() thread was not woken up");
+
+int      cryptoretwait = 0;
+module_param(cryptoretwait, int, 0444);
+
+int      cryptoretnow = 0;
+module_param(cryptoretnow, int, 0444);
+MODULE_PARM_DESC(cryptoretnow, "how many times we did the callback directly");
+
+int      cryptoproc_generation=0;
+module_param(cryptoproc_generation, int, 0444);
+
+uint      cryptoproc_submitted=0;
+module_param(cryptoproc_submitted, uint, 0444);
+
+int      cryptoretproc_generation=0;
+module_param(cryptoretproc_generation, int, 0444);
+
 static	int crypto_proc(void *arg);
 static	int crypto_ret_proc(void *arg);
 static	int crypto_invoke(struct cryptop *crp, int hint);
@@ -179,6 +279,19 @@ static  int crypto_init(void);
 
 static	struct cryptostats cryptostats;
 
+module_param_named(cs_ops,      cryptostats.cs_ops,      int,0444);
+module_param_named(cs_ops_done, cryptostats.cs_ops_done, int,0444);
+module_param_named(cs_blocks,   cryptostats.cs_blocks,   int,0444);
+module_param_named(cs_drops,    cryptostats.cs_drops,    uint,0444);
+unsigned int *ocf_cs_ops = &cryptostats.cs_ops;
+unsigned int *ocf_cs_ops_done = &cryptostats.cs_ops_done;
+unsigned int *ocf_cs_blocks = &cryptostats.cs_blocks;
+
+
+#if defined(CONFIG_OCF_PROFILE)
+static cryptop_ts_t crypto_proc_thread_times;
+static cryptop_ts_t crypto_return_thread_times;
+#endif
 
 /*
  * Create a new session.
@@ -192,7 +305,7 @@ crypto_newsession(u_int64_t *sid, struct cryptoini *cri, enum cryptodev_selectio
 	unsigned long d_flags;
 	struct cryptocap *cap = NULL;
 
-	if (!crypto_initted) {
+        if (atomic_read (&crypto_needs_init) < 0) {
 		int i = crypto_init();
 		if (i) {
 			printk("crypto: failed to init crypto (%d)!\n", i);
@@ -332,6 +445,7 @@ crypto_newsession(u_int64_t *sid, struct cryptoini *cri, enum cryptodev_selectio
 		err = (*cap->cc_newsession)(cap->cc_arg, &lid, cri);
 	}
 	
+	dprintk("%s - hid=%u new lid=%u\n", __FUNCTION__, besthid, lid);
 	if (err == 0) {
 		/* XXX assert (hid &~ 0xffffff) == 0 */
 		/* XXX assert (cap->cc_flags &~ 0xff) == 0 */
@@ -427,7 +541,7 @@ crypto_get_driverid(u_int32_t flags, const char *drivername)
 
 	dprintk("%s()\n", __FUNCTION__);
 
-	if (!crypto_initted) {
+        if (atomic_read (&crypto_needs_init) < 0) {
 		int rc = crypto_init();
 		if (rc) {
 			printk("crypto: failed to init crypto (%d)!\n", rc);
@@ -905,21 +1019,31 @@ crypto_unblock(u_int32_t driverid, int what)
 	int needwakeup, err;
 	unsigned long q_flags;
 
+	debugfs_circular_stamp(crypto_proc_circ, 0x00003333);
+	cryptoproc_unblock_called++;
 	dprintk("%s()\n", __FUNCTION__);
 	CRYPTO_Q_LOCK();
 	cap = crypto_checkdriver(driverid);
 	if (cap != NULL) {
 		needwakeup = 0;
+
 		if (what & CRYPTO_SYMQ) {
-			needwakeup |= cap->cc_qblocked;
-			cap->cc_qblocked = 0;
+			needwakeup = 1;
+			cap->cc_qblocked++;
 		}
 		if (what & CRYPTO_ASYMQ) {
-			needwakeup |= cap->cc_kqblocked;
+			needwakeup = 1;
 			cap->cc_kqblocked = 0;
 		}
-		if (needwakeup)
+		if (needwakeup) {
+                        cryptop_ts_record (&crypto_proc_thread_times, OCF_TS_PTH_WAKE);
+			cp_printk("crypto proc wake up\n");
+			cryptoproc_active++;
+			cryptoproc_unblock++;
 			wake_up_interruptible(&cryptoproc_wait);
+                } else {
+			cryptoproc_unblock_unneeded++;
+		}
 		err = 0;
 	} else
 		err = EINVAL;
@@ -929,7 +1053,10 @@ crypto_unblock(u_int32_t driverid, int what)
 }
 
 /*
- * Add a crypto request to a queue, to be processed by the kernel thread.
+ * crypto_dispatch - Add a crypto request to a crypto queue.
+ * @crp - the already formatted request
+ *
+ * Returns 0 upon success, and errno on fail.
  */
 int
 crypto_dispatch(struct cryptop *crp)
@@ -938,10 +1065,13 @@ crypto_dispatch(struct cryptop *crp)
 	int result;
 	unsigned long q_flags;
 	struct cryptocap *cap;
+	int q_cnt;
+	static int stamp1000done=0;
 
 	dprintk("%s()\n", __FUNCTION__);
 
 	cryptostats.cs_ops++;
+        cryptop_ts_record (&crp->crp_times, OCF_TS_CRP_DISPATCH);
 
 	if (atomic_read(&crypto_q_cnt) >= crypto_q_max) {
 		cryptostats.cs_drops++;
@@ -949,6 +1079,21 @@ crypto_dispatch(struct cryptop *crp)
 	}
 	atomic_inc(&crypto_q_cnt);
 
+	q_cnt=atomic_read(&crypto_q_cnt);
+	if(q_cnt == crypto_q_max) {
+		/* X-OFF */
+		if(!stamp1000done) {
+			debugfs_circular_stamp(crypto_proc_circ, 0x22004444);
+		}
+		stamp1000done=1;
+	} else {
+		/* X-ON */
+		if(stamp1000done) {
+			debugfs_circular_stamp(crypto_proc_circ, 0x11004444);
+		}
+		stamp1000done=0;
+	}
+
 	cap = crypto_checkdriver(hid);
 
 	CRYPTO_Q_LOCK();
@@ -963,7 +1108,7 @@ crypto_dispatch(struct cryptop *crp)
 		 * immediately; dispatch it directly to the
 		 * driver unless the driver is currently blocked.
 		 */
-		if (cap && !cap->cc_qblocked) {
+		if (cap && cap->cc_qblocked>=0) {
 			CRYPTO_Q_UNLOCK();
 			result = crypto_invoke(crp, 0);
 			CRYPTO_Q_LOCK();
@@ -977,8 +1122,9 @@ crypto_dispatch(struct cryptop *crp)
 				 * order is preserved but this can place them
 				 * behind batch'd ops.
 				 */
-				crypto_drivers[hid].cc_qblocked = 1;
+				crypto_drivers[hid].cc_qblocked--;
 				list_add_tail(&crp->crp_list, &crp_q);
+				crypto_qs++;
 				cryptostats.cs_blocks++;
 				result = 0;
 			}
@@ -988,20 +1134,23 @@ crypto_dispatch(struct cryptop *crp)
 			 * it unblocks and the kernel thread gets kicked.
 			 */
 			list_add_tail(&crp->crp_list, &crp_q);
+			crypto_qs++;
 			result = 0;
 		}
 	} else {
-		int wasempty;
 		/*
 		 * Caller marked the request as ``ok to delay'';
 		 * queue it for the dispatch thread.  This is desirable
 		 * when the operation is low priority and/or suitable
 		 * for batching.
 		 */
-		wasempty = list_empty(&crp_q);
 		list_add_tail(&crp->crp_list, &crp_q);
-		if (wasempty)
-			wake_up_interruptible(&cryptoproc_wait);
+		crypto_qs++;
+		cryptop_ts_record (&crypto_proc_thread_times, OCF_TS_PTH_WAKE);
+		cp_printk("crypto proc wake up 2\n");
+		cryptoproc_active++;
+		debugfs_circular_stamp(crypto_proc_circ, 0x00001111|(q_cnt<<20));
+		wake_up_interruptible(&cryptoproc_wait);
 		result = 0;
 	}
 	if (result != 0)
@@ -1114,6 +1263,7 @@ crypto_kinvoke(struct cryptkop *krp, int hint)
 static int
 crypto_invoke(struct cryptop *crp, int hint)
 {
+        int rc;
 	u_int32_t hid;
 	int (*process)(void*, struct cryptop *, int);
 
@@ -1138,8 +1288,8 @@ crypto_invoke(struct cryptop *crp, int hint)
 			crypto_freesession(crp->crp_sid);
 		process = crypto_drivers[hid].cc_process;
 	} else {
-		printk("%s() found hid(%d) >= crypto_drivers_num(%d)\n",
-			__FUNCTION__, hid, crypto_drivers_num);
+		printk("%s() {%p} found hid(%d) >= crypto_drivers_num(%d)\n",
+		       __FUNCTION__, crp, hid, crypto_drivers_num);
 		process = NULL;
 	}
 
@@ -1165,7 +1315,14 @@ crypto_invoke(struct cryptop *crp, int hint)
 		/*
 		 * Invoke the driver to process the request.
 		 */
-		return (*process)(crypto_drivers[hid].cc_arg, crp, hint);
+                cryptop_ts_record (&crp->crp_times, OCF_TS_CRP_PROCESS);
+
+		rc = (*process)(crypto_drivers[hid].cc_arg, crp, hint);
+
+                /* NOTE: we cannot touch crp at this point because it could have
+                 * already finished and been deleted in another context */
+
+                return rc;
 	}
 }
 
@@ -1185,6 +1342,9 @@ crypto_freereq(struct cryptop *crp)
 		kmem_cache_free(cryptodesc_zone, crd);
 	}
 
+        cryptop_ts_record (&crp->crp_times, OCF_TS_CRP_DESTROY);
+        cryptop_ts_process (&crp->crp_times);
+
 	kmem_cache_free(cryptop_zone, crp);
 }
 
@@ -1200,6 +1360,7 @@ crypto_getreq(int num)
 	crp = kmem_cache_alloc(cryptop_zone, SLAB_ATOMIC);
 	if (crp != NULL) {
 		memset(crp, 0, sizeof(*crp));
+                cryptop_ts_reset (&crp->crp_times);
 		INIT_LIST_HEAD(&crp->crp_list);
 		init_waitqueue_head(&crp->crp_waitq);
 		while (num--) {
@@ -1222,6 +1383,8 @@ crypto_getreq(int num)
 void
 crypto_done(struct cryptop *crp)
 {
+        cryptop_ts_record (&crp->crp_times, OCF_TS_CRP_DONE);
+
 	dprintk("%s()\n", __FUNCTION__);
 	if ((crp->crp_flags & CRYPTO_F_DONE) == 0) {
 		crp->crp_flags |= CRYPTO_F_DONE;
@@ -1229,6 +1392,8 @@ crypto_done(struct cryptop *crp)
 	} else
 		printk("crypto: crypto_done op already done, flags 0x%x",
 				crp->crp_flags);
+
+	cryptostats.cs_ops_done++;
 	if (crp->crp_etype != 0)
 		cryptostats.cs_errs++;
 	/*
@@ -1246,6 +1411,7 @@ crypto_done(struct cryptop *crp)
 		 * callback routine does very little (e.g. the
 		 * /dev/crypto callback method just does a wakeup).
 		 */
+		cryptoretnow++;
 		crp->crp_callback(crp);
 	} else {
 		int wasempty;
@@ -1255,10 +1421,13 @@ crypto_done(struct cryptop *crp)
 		 */
 		CRYPTO_RETQ_LOCK();
 		wasempty = list_empty(&crp_ret_q);
+		cryptoretwait++;
 		list_add_tail(&crp->crp_list, &crp_ret_q);
 
-		if (wasempty)
+		if (wasempty) {
+                        cryptop_ts_record (&crypto_return_thread_times, OCF_TS_RTH_WAKE);
 			wake_up_interruptible(&cryptoretproc_wait);	/*shared wait channel */
+                }
 		CRYPTO_RETQ_UNLOCK();
 	}
 }
@@ -1296,10 +1465,13 @@ crypto_kdone(struct cryptkop *krp)
 		 */
 		CRYPTO_RETQ_LOCK();
 		wasempty = list_empty(&crp_ret_kq);
+		cryptoretwait++;
 		list_add_tail(&krp->krp_list, &crp_ret_kq);
 
-		if (wasempty)
+		if (wasempty) {
+                        cryptop_ts_record (&crypto_return_thread_times, OCF_TS_RTH_WAKE);
 			wake_up_interruptible(&cryptoretproc_wait);/* shared wait channel */
+                }
 		CRYPTO_RETQ_UNLOCK();
 	}
 }
@@ -1332,17 +1504,55 @@ out:
 	return (0);
 }
 
+
+int crypto_proc_somework(void)
+{
+    struct cryptop *crp, *crpn;
+    struct cryptkop *krpp;
+    struct cryptocap *cap;
+    int somework = true;
+    unsigned long d_flags;
+
+    CRYPTO_DRIVER_LOCK();
+    list_for_each_entry_safe(crp, crpn, &crp_q, crp_list) {
+	    u_int32_t hid = CRYPTO_SESID2HID(crp->crp_sid);
+	    cap = crypto_checkdriver(hid);
+
+	    if(!cap) continue;
+
+	    if(cap->cc_qblocked >= 0) goto out;
+    }
+
+    list_for_each_entry(krpp, &crp_kq, krp_list) {
+	    cap = crypto_checkdriver(krpp->krp_hid);
+
+	    if(!cap) continue;
+
+	    if (!cap->cc_kqblocked) goto out;
+    }
+
+    cryptoproc_falsewakes++;
+    somework=false;
+
+out:
+    CRYPTO_DRIVER_UNLOCK();
+    return somework;
+}
+
+
+
 /*
  * Crypto thread, dispatches crypto requests.
  */
 static int
 crypto_proc(void *arg)
 {
-	struct cryptop *crp, *submit;
+    struct cryptop *crp, *crpn, *submit, *lastsubmit= NULL;
 	struct cryptkop *krp, *krpp;
 	struct cryptocap *cap;
 	int result, hint;
 	unsigned long q_flags;
+	int wakeonce1000=1;
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
 	daemonize();
@@ -1354,6 +1564,8 @@ crypto_proc(void *arg)
 #else
 	daemonize("crypto");
 #endif
+	current->rt_priority=99;
+	current->policy=SCHED_RR;
 
 	CRYPTO_Q_LOCK();
 	for (;;) {
@@ -1364,16 +1576,31 @@ crypto_proc(void *arg)
 		 */
 		submit = NULL;
 		hint = 0;
-		list_for_each_entry(crp, &crp_q, crp_list) {
+
+		if (atomic_read(&crypto_q_cnt) != 1000) {
+			int num=atomic_read(&crypto_q_cnt) << 20;
+			debugfs_circular_stamp(crypto_proc_circ, 0x00002222|num);
+		}
+
+		list_for_each_entry_safe(crp, crpn, &crp_q, crp_list) {
 			u_int32_t hid = CRYPTO_SESID2HID(crp->crp_sid);
 			cap = crypto_checkdriver(hid);
+			
+			cp_printk("found hid=%u with cap=%p, w/cc_process=%p\n",
+				  hid, cap, cap?cap->cc_process:NULL);
+			
 			if (cap == NULL || cap->cc_process == NULL) {
 				/* Op needs to be migrated, process it. */
-				if (submit == NULL)
-					submit = crp;
-				break;
+			    if (submit == NULL) {
+				    cp_printk("submit was NULL, so submit=crpp=%p\n", crp);
+				submit = crp;
+			    }
+			    break;
 			}
-			if (!cap->cc_qblocked) {
+
+			cp_printk("cap was != NULL, crp=%p and cc_qblocked=%u\n", 
+				   crp, cap->cc_qblocked);
+			if (cap->cc_qblocked >= 0) {
 				if (submit != NULL) {
 					/*
 					 * We stop on finding another op,
@@ -1383,21 +1610,39 @@ crypto_proc(void *arg)
 					 * better to just use a per-driver
 					 * queue instead.
 					 */
-					if (CRYPTO_SESID2HID(submit->crp_sid) == hid)
-						hint = CRYPTO_HINT_MORE;
-					break;
+
+				    cp_printk("submit=%p, found another op, looking for more hint\n", submit);
+
+				    if (CRYPTO_SESID2HID(submit->crp_sid) == hid)
+					    hint = CRYPTO_HINT_MORE;
+				    break;
 				} else {
 					submit = crp;
+					cp_printk("submit=crp=%p, with batch=%u\n", submit, ((submit->crp_flags & CRYPTO_F_BATCH) == 0));
 					if ((submit->crp_flags & CRYPTO_F_BATCH) == 0)
 						break;
 					/* keep scanning for more are q'd */
 				}
+			} else {
+			    /* queue was blocked, leave item here */
+			    cp_printk("cc_qblocked=%u crp=%p left on list\n", 
+				       cap->cc_qblocked, crp);
 			}
 		}
+
 		if (submit != NULL) {
+			if (cryptoproc_active==0) cryptoproc_lastditch++;
+			cryptoproc_active=1;
+
+			lastsubmit = submit;
 			list_del(&submit->crp_list);
+			crypto_qs--;
+                        cryptop_ts_record (&crypto_proc_thread_times, OCF_TS_PTH_INVOKE);
+			debugfs_circular_stamp(crypto_proc_circ, 0x00002224);
 			CRYPTO_Q_UNLOCK();
+			cryptoproc_submitted++;
 			result = crypto_invoke(submit, hint);
+			debugfs_circular_stamp(crypto_proc_circ, 0x00002244);
 			CRYPTO_Q_LOCK();
 			if (result == ERESTART) {
 				/*
@@ -1410,9 +1655,13 @@ crypto_proc(void *arg)
 				 * it at the end does not work.
 				 */
 				/* XXX validate sid again? */
-				crypto_drivers[CRYPTO_SESID2HID(submit->crp_sid)].cc_qblocked = 1;
+				crypto_drivers[CRYPTO_SESID2HID(submit->crp_sid)].cc_qblocked--;
+				debugfs_circular_stamp(crypto_proc_circ, 0x00007777);
+				
 				list_add(&submit->crp_list, &crp_q);
+				crypto_qs++;
 				cryptostats.cs_blocks++;
+				submit=NULL;
 			}
 		}
 
@@ -1450,9 +1699,10 @@ crypto_proc(void *arg)
 				list_add(&krp->krp_list, &crp_kq);
 				cryptostats.cs_kblocks++;
 			}
+			continue;
 		}
 
-		if (submit == NULL && krp == NULL) {
+		if (submit == NULL && krp == NULL && cryptoproc_active==0) {
 			/*
 			 * Nothing more to be processed.  Sleep until we're
 			 * woken because there are more ops to process.
@@ -1466,26 +1716,66 @@ crypto_proc(void *arg)
 			 * and some become blocked while others do not.
 			 */
 			dprintk("%s - sleeping\n", __FUNCTION__);
+                        cryptop_ts_record (&crypto_proc_thread_times, OCF_TS_PTH_SLEEP);
+                        cryptop_ts_process (&crypto_proc_thread_times);
+                        cryptop_ts_reset (&crypto_proc_thread_times);
+
+			cp_printk("crypto proc sleep: %u/%u\n", cryptoproc_active, cryptoproc_generation);
 			CRYPTO_Q_UNLOCK();
-			wait_event_interruptible(cryptoproc_wait,
-					cryptoproc == (pid_t) -1 ||
-					!list_empty(&crp_q) ||
-					!list_empty(&crp_kq));
-			if (signal_pending (current)) {
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
-				spin_lock_irq(&current->sigmask_lock);
-#endif
-				flush_signals(current);
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
-				spin_unlock_irq(&current->sigmask_lock);
-#endif
+
+			wait_event_interruptible_timeout(cryptoproc_wait,
+							 cryptoproc == (pid_t) -1 ||
+							 (atomic_read(&crypto_q_cnt) >= 0						&& crypto_proc_somework()) ||
+							 cryptoproc_active>0,
+							 msecs_to_jiffies(10000));
+
+			if (atomic_read(&crypto_q_cnt) != 1000) {
+				debugfs_circular_stamp(crypto_proc_circ, 0x00002223);
+				wakeonce1000=1;
+			} else {
+				if(!wakeonce1000) {
+					wakeonce1000=0;
+					debugfs_circular_stamp(crypto_proc_circ, 0x00002233);
+				}
 			}
+
+			if (signal_pending (current)) {
+				flush_signals(current);
+			}
+
+			if(crypto_proc_debug) {
+			    static int print_count=0;
+
+			    if(print_count++ > 128) {
+				if(printk_ratelimit()) {
+				    printk("%s - awake cryptoproc=%d crp_q=%u crp_kq=%u cryptoproc_active=%u\n", __FUNCTION__,
+					   cryptoproc,
+					   !list_empty(&crp_q),
+					   !list_empty(&crp_kq), cryptoproc_active);
+				}
+			    }
+			}
+
 			CRYPTO_Q_LOCK();
-			dprintk("%s - awake\n", __FUNCTION__);
+			cryptoproc_generation++;
+
 			if (cryptoproc == (pid_t) -1)
 				break;
 			cryptostats.cs_intrs++;
+
+                        // we are now running
+                        cryptop_ts_record (&crypto_proc_thread_times, OCF_TS_PTH_RUN);
 		}
+
+		/*
+		 * look once more for things to do, because they might
+		 * have shown up while we were calling the invoke function.
+		 */
+		if (submit == NULL && krp == NULL && cryptoproc_active>0) {
+			cryptoproc_trylastditch++;
+			cryptoproc_active=0;
+		}
+
 	}
 	CRYPTO_Q_UNLOCK();
 	complete_and_exit(&cryptoproc_exited, 0);
@@ -1517,11 +1807,15 @@ crypto_ret_proc(void *arg)
 	CRYPTO_RETQ_LOCK();
 	for (;;) {
 		/* Harvest return q's for completed ops */
+		cryptoretproc_generation++;
+
 		crpt = NULL;
 		if (!list_empty(&crp_ret_q))
 			crpt = list_entry(crp_ret_q.next, typeof(*crpt), crp_list);
-		if (crpt != NULL)
+		if (crpt != NULL) {
+			cryptoretwait--;
 			list_del(&crpt->crp_list);
+		}
 
 		krpt = NULL;
 		if (!list_empty(&crp_ret_kq))
@@ -1530,6 +1824,7 @@ crypto_ret_proc(void *arg)
 			list_del(&krpt->krp_list);
 
 		if (crpt != NULL || krpt != NULL) {
+                        cryptop_ts_record (&crypto_return_thread_times, OCF_TS_RTH_CALLBACK);
 			CRYPTO_RETQ_UNLOCK();
 			/*
 			 * Run callbacks unlocked.
@@ -1545,19 +1840,16 @@ crypto_ret_proc(void *arg)
 			 * woken because there are more returns to process.
 			 */
 			dprintk("%s - sleeping\n", __FUNCTION__);
+                        cryptop_ts_record (&crypto_return_thread_times, OCF_TS_RTH_SLEEP);
+                        cryptop_ts_process (&crypto_return_thread_times);
+                        cryptop_ts_reset (&crypto_return_thread_times);
 			CRYPTO_RETQ_UNLOCK();
 			wait_event_interruptible(cryptoretproc_wait,
 					cryptoretproc == (pid_t) -1 ||
 					!list_empty(&crp_ret_q) ||
 					!list_empty(&crp_ret_kq));
 			if (signal_pending (current)) {
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
-				spin_lock_irq(&current->sigmask_lock);
-#endif
 				flush_signals(current);
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
-				spin_unlock_irq(&current->sigmask_lock);
-#endif
 			}
 			CRYPTO_RETQ_LOCK();
 			dprintk("%s - awake\n", __FUNCTION__);
@@ -1566,6 +1858,9 @@ crypto_ret_proc(void *arg)
 				break;
 			}
 			cryptostats.cs_rets++;
+
+                        // we are now running
+                        cryptop_ts_record (&crypto_return_thread_times, OCF_TS_RTH_RUN);
 		}
 	}
 	CRYPTO_RETQ_UNLOCK();
@@ -1580,10 +1875,17 @@ crypto_init(void)
 
 	dprintk("%s(0x%x)\n", __FUNCTION__, (int) crypto_init);
 
-	if (crypto_initted)
-		return 0;
-	crypto_initted = 1;
+        if (! atomic_inc_and_test (&crypto_needs_init)) {
+                // we were already initalized, restore counter and get out
+                atomic_dec (&crypto_needs_init);
+                return 0;
+        }
 
+        cryptop_ts_init ();
+        cryptop_ts_reset (&crypto_proc_thread_times);
+        cryptop_ts_reset (&crypto_return_thread_times);
+
+	crypto_proc_circ = debugfs_create_circular("ocfops", 0444, NULL);
 	atomic_set(&crypto_q_cnt, 0);
 
 	spin_lock_init(&crypto_drivers_lock);
@@ -1679,6 +1981,8 @@ crypto_exit(void)
 		kmem_cache_destroy(cryptodesc_zone);
 	if (cryptop_zone != NULL)
 		kmem_cache_destroy(cryptop_zone);
+
+        cryptop_ts_exit ();
 }
 
 
