@@ -49,7 +49,8 @@
 #include <linux/poll.h>
 #include <linux/random.h>
 #include <linux/jiffies.h>
-#include <crypto/cryptodev.h>
+#include <opencrypto/crypto.h>
+#include <opencrypto/cryptodev.h>
 
 #ifdef FIPS_TEST_RNG
 #include "rndtest.h"
