@@ -119,17 +119,6 @@ extern int sysctl_ipsec_debug_verbose;
 #endif /* CONFIG_KLIPS_DEBUG */
 #endif /* __KERNEL__ */
 
-#ifdef CONFIG_KLIPS_DEBUG
-#define DB_TN_INIT	0x0001
-#define DB_TN_PROCFS	0x0002
-#define DB_TN_XMIT	0x0010
-#define DB_TN_OHDR	0x0020
-#define DB_TN_CROUT	0x0040
-#define DB_TN_OXFS	0x0080
-#define DB_TN_REVEC	0x0100
-#define DB_TN_ENCAP     0x0200
-#endif /* CONFIG_KLIPS_DEBUG */
-
 // manage ipsec xmit state objects
 extern int ipsec_xmit_state_cache_init (void);
 extern void ipsec_xmit_state_cache_cleanup (void);
