@@ -107,8 +107,10 @@ static inline const char *auth_name_id (unsigned id) {
 #define IPS_XFORM_NAME(x) \
 	PROTO2TXT((x)->ips_said.proto), \
 	(x)->ips_said.proto == IPPROTO_COMP ? \
-		((x)->ips_encalg == SADB_X_CALG_DEFLATE ? \
-		 "_DEFLATE" : "_UNKNOWN_comp") : \
+		((x)->ips_compalg == SADB_X_CALG_NONE ? "_NONE" : 	\
+		 (x)->ips_compalg == SADB_X_CALG_DEFLATE ? "_DEFLATE":	\
+		 (x)->ips_compalg == SADB_X_CALG_LZS ? "_LZS" :		\
+		 "_UNKNOWN_comp") :					\
 	(x)->ips_encalg == ESP_NONE ? "" : \
 	(x)->ips_encalg == ESP_3DES ? "_3DES" : \
 	(x)->ips_encalg == ESP_AES ? "_AES" : \
