@@ -1,5 +1,5 @@
 
-#include <sys/queue.h>
+#include <sysqueue.h>
 #include <linux/types.h>
 #define u8 __u8
 
