@@ -66,6 +66,8 @@
 #include "ocf_cryptodev.h"
 #endif
 
+TAILQ_HEAD(req_queue, pluto_crypto_req_cont);
+
 struct pluto_crypto_worker {
     int   pcw_helpernum;
     pid_t pcw_pid;
@@ -75,12 +77,11 @@ struct pluto_crypto_worker {
     int   pcw_maxcritwork;  /* how many critical things can be queued */
     bool  pcw_dead;         /* worker is dead, waiting for reap */
     bool  pcw_reaped;       /* worker has been reaped, waiting for dealloc */
-    struct pluto_crypto_req_cont *pcw_cont;
+    struct req_queue pcw_active;
 };
 
-static struct pluto_crypto_req_cont *backlogqueue;
-static struct pluto_crypto_req_cont **backlogqueue_last = &backlogqueue;
-static int                           backlogqueue_len;
+static struct req_queue backlog;
+static int       backlogqueue_len=0;
 
 static void init_crypto_helper(struct pluto_crypto_worker *w, int n);
 static void cleanup_crypto_helper(struct pluto_crypto_worker *w, int status);
@@ -90,7 +91,7 @@ extern void free_preshared_secrets(void);
 /* may be NULL if we are to do all the work ourselves */
 struct pluto_crypto_worker *pc_workers = NULL;
 int pc_workers_cnt = 0;
-int pc_worker_num;
+int pc_worker_num;  /* count of highest child */
 pcr_req_id pcw_id;
 
 bool using_vulcan_hack=FALSE;
@@ -179,7 +180,7 @@ helper_passert_fail(const char *pred_str
 }
 
 
-void pluto_crypto_helper(int fd, int helpernum)
+void pluto_crypto_helper(int fd)
 {
     FILE *in  = fdopen(fd, "rb");
     FILE *out = fdopen(fd, "wb");
@@ -189,12 +190,11 @@ void pluto_crypto_helper(int fd, int helpernum)
     signal(SIGHUP, catchhup);
     signal(SIGUSR1, catchusr1);
 
-    pc_worker_num = helpernum;
     /* make us lower priority that average */
     setpriority(PRIO_PROCESS, 0, 10);
 
     DBG(DBG_CONTROL, DBG_log("helper %d waiting on fd: %d"
-			     , helpernum, fileno(in)));
+			     , pc_helper_num, fileno(in)));
 
     memset(reqbuf, 0, sizeof(reqbuf));
     while(fread((char*)reqbuf, sizeof(r->pcr_len), 1, in) == 1) {
@@ -212,7 +212,7 @@ void pluto_crypto_helper(int fd, int helpernum)
 	/* okay, got a basic size, read the rest of it */
 
 	DBG(DBG_CONTROL, DBG_log("helper %d read %d+4/%d bytesfd: %d"
-				 , helpernum, actnum, r->pcr_len, fileno(in)));
+				 , pc_helper_num, actnum, r->pcr_len, fileno(in)));
 
 	if(actnum != restlen) {
 	    /* faulty read. die, parent will restart us */
@@ -242,7 +242,7 @@ void pluto_crypto_helper(int fd, int helpernum)
     }
 
     if(!feof(in)) {
-	loglog(RC_LOG_SERIOUS, "helper %d got error: %s", helpernum, strerror(ferror(in)));
+	loglog(RC_LOG_SERIOUS, "helper %d got error: %s", pc_helper_num, strerror(ferror(in)));
     }
 
     /* probably normal EOF */
@@ -374,10 +374,8 @@ err_t send_crypto_helper_request(struct pluto_crypto_req *r
 
     if(cnt == 0 && r->pcr_pcim >= pcim_demand_crypto) {
 	/* it is very important. Put it all on a queue for later */
-	if(!backlogqueue_last) backlogqueue_last = &backlogqueue;
-	*backlogqueue_last = cn;
-	cn->pcrc_next = NULL;
-	backlogqueue_last  = &cn->pcrc_next;
+	
+	TAILQ_INSERT_TAIL(&backlog, cn, pcrc_list);
 
 	/* copy the request */
 	r = clone_bytes(r, r->pcr_len, "saved cryptorequest");
@@ -413,8 +411,7 @@ err_t send_crypto_helper_request(struct pluto_crypto_req *r
     }
 
     /* link it to the active worker list */
-    cn->pcrc_next = w->pcw_cont;
-    w->pcw_cont = cn;
+    TAILQ_INSERT_TAIL(&w->pcw_active, cn, pcrc_list);
 
     passert(w->pcw_pid != -1);
     passert(w->pcw_pipe != -1);
@@ -440,10 +437,11 @@ static void crypto_send_backlog(struct pluto_crypto_worker *w)
     struct pluto_crypto_req_cont *cn;
 
     if(backlogqueue_len > 0) {
-	passert(backlogqueue != NULL);
+
+	passert(backlog.tqh_first != NULL);
+	cn = backlog.tqh_first;
+	TAILQ_REMOVE(&backlog, cn, pcrc_list);
 	
-	cn = backlogqueue;
-	backlogqueue = cn->pcrc_next;
 	backlogqueue_len--;
 	
 	r = cn->pcrc_pcr;
@@ -465,8 +463,7 @@ static void crypto_send_backlog(struct pluto_crypto_worker *w)
 	}
 	
 	/* link it to the active worker list */
-	cn->pcrc_next = w->pcw_cont;
-	w->pcw_cont = cn;
+	TAILQ_INSERT_TAIL(&w->pcw_active, cn, pcrc_list);
 	
 	passert(w->pcw_pid != -1);
 	passert(w->pcw_pipe != -1);
@@ -513,15 +510,12 @@ void delete_cryptographic_continuation(struct state *st)
 
     for(i=0; i<pc_workers_cnt; i++) {
 	struct pluto_crypto_worker *w = &pc_workers[i];
-	struct pluto_crypto_req_cont *cn, **cnp;
+	struct pluto_crypto_req_cont *cn;
 
-	cn = w->pcw_cont;
-	cnp = &w->pcw_cont;
-	while(cn && st->st_serialno != cn->pcrc_serialno) {
-	    cnp = &cn->pcrc_next;
-	    cn = cn->pcrc_next;
-	}
-	
+	for(cn = w->pcw_active.tqh_first;
+	    cn!=NULL && st->st_serialno != cn->pcrc_serialno;
+	    cn = cn->pcrc_list.tqe_next);
+		
 	if(cn == NULL) {
 	    DBG(DBG_CRYPT, DBG_log("no suspended cryptographic state for %lu\n"
 				   , st->st_serialno));
@@ -529,8 +523,7 @@ void delete_cryptographic_continuation(struct state *st)
 	}
 
 	/* unlink it, and free it */
-	*cnp = cn->pcrc_next;
-	cn->pcrc_next = NULL;
+	TAILQ_REMOVE(&w->pcw_active, cn, pcrc_list);
  
 	if(cn->pcrc_free) {
 	    /*
@@ -561,7 +554,7 @@ void handle_helper_comm(struct pluto_crypto_worker *w)
     struct pluto_crypto_req *r;
     int restlen;
     int actlen;
-    struct pluto_crypto_req_cont *cn, **cnp;
+    struct pluto_crypto_req_cont *cn;
 
     DBG(DBG_CRYPT|DBG_CONTROL
 	, DBG_log("helper %u has finished work (cnt now %d)"
@@ -635,13 +628,10 @@ void handle_helper_comm(struct pluto_crypto_worker *w)
     crypto_send_backlog(w);
 
     /* now match up request to continuation, and invoke it */
-    cn = w->pcw_cont;
-    cnp = &w->pcw_cont;
-    while(cn && r->pcr_id != cn->pcrc_id) {
-	cnp = &cn->pcrc_next;
-	cn = cn->pcrc_next;
-    }
-
+    for(cn = w->pcw_active.tqh_first;
+	cn!=NULL && r->pcr_id != cn->pcrc_id;
+	cn = cn->pcrc_list.tqe_next);
+		
     if(cn == NULL) {
 	loglog(RC_LOG_SERIOUS
 	       , "failed to find continuation associated with req %u\n",
@@ -650,8 +640,7 @@ void handle_helper_comm(struct pluto_crypto_worker *w)
     }
 
     /* unlink it */
-    *cnp = cn->pcrc_next;
-    cn->pcrc_next = NULL;
+    TAILQ_REMOVE(&w->pcw_active, cn, pcrc_list);
  
     passert(cn->pcrc_func != NULL);
 
@@ -690,6 +679,7 @@ static void init_crypto_helper(struct pluto_crypto_worker *w, int n)
     w->pcw_work     = 0;
     w->pcw_reaped = FALSE;
     w->pcw_dead   = FALSE;
+    TAILQ_INIT(&w->pcw_active);
 
     /* set the send/received queue length to be at least maxcritwork
      * times sizeof(pluto_crypto_req) in size
@@ -766,7 +756,7 @@ static void init_crypto_helper(struct pluto_crypto_worker *w, int n)
 #ifdef HAVE_OCF_AND_OPENSSL
 	load_cryptodev();
 #endif
-	free_preshared_secrets();
+	free_preshared_secrets();  /* get rid of sensitive data */
 	openswan_passert_fail = helper_passert_fail;
 	debug_prefix='!';
 
@@ -778,7 +768,7 @@ static void init_crypto_helper(struct pluto_crypto_worker *w, int n)
 	}
 #endif	
 
-	pluto_crypto_helper(fds[1], n);
+	pluto_crypto_helper(fds[1]);
 
 #ifdef VULCAN_PK
 	if(using_vulcan_hack) {
@@ -849,6 +839,8 @@ void init_crypto_helpers(int nhelpers)
     pc_workers_cnt = 0;
     pcw_id = 1;
 
+    TAILQ_INIT(&backlog);
+
     cryptodev.calc_dh_shared = calc_dh_shared_gmp;
 #ifdef VULCAN_PK
     if(please_use_vulcan_hack) {
