@@ -160,6 +160,7 @@ usage(const char *mess)
 	    " [--debug-x509]"
 	    " [--debug-dns]"
 	    " [--debug-oppo]"
+	    " [--debug-oppoinfo]"
 	    " [--debug-dpd]"
 	    " [ --debug-private]"
 	    " [ --debug-pfkey]"
@@ -386,6 +387,7 @@ main(int argc, char **argv)
 	    { "debug-klips", no_argument, NULL, DBG_KLIPS + DBG_OFFSET },
 	    { "debug-dns", no_argument, NULL, DBG_DNS + DBG_OFFSET },
 	    { "debug-oppo", no_argument, NULL, DBG_OPPO + DBG_OFFSET },
+	    { "debug-oppoinfo", no_argument, NULL, DBG_OPPOINFO + DBG_OFFSET },
 	    { "debug-controlmore", no_argument, NULL, DBG_CONTROLMORE + DBG_OFFSET },
 	    { "debug-dpd", no_argument, NULL, DBG_DPD + DBG_OFFSET },
             { "debug-x509", no_argument, NULL, DBG_X509 + DBG_OFFSET },
@@ -780,14 +782,14 @@ main(int argc, char **argv)
         const char *v = init_pluto_vendorid();
 	const char *vc = ipsec_version_code();
 
-        openswan_log("Starting Pluto (Openswan Version %s%s; Vendor ID %s)"
-            , vc
-            , compile_time_interop_options
-            , v);
+        openswan_log("Starting Pluto (Openswan Version %s%s; Vendor ID %s) pid:%u"
+		     , vc
+		     , compile_time_interop_options
+		     , v, getpid());
 #else
-        openswan_log("Starting Pluto (Openswan Version %s%s)"
-            , ipsec_version_code()
-            , compile_time_interop_options);
+        openswan_log("Starting Pluto (Openswan Version %s%s) pid:%u"
+		     , ipsec_version_code()
+		     , compile_time_interop_options, getpid());
 #endif
 
 	if((vc[0]=='c' && vc[1]=='v' && vc[2]=='s') ||
