@@ -105,6 +105,8 @@ else
 fi
 
 mkdir -p $POOLSPACE
+if [ ! -d ${OPENSWANSRCDIR}/UMLPOOL/. ]; then ln -s $POOLSPACE ${OPENSWANSRCDIR}/UMLPOOL; fi
+
 UMLMAKE=$POOLSPACE/Makefile
 NOW=`date`
 USER=${USER-`id -un`}
@@ -144,7 +146,8 @@ done
 if $NEED_plain && [ ! -x $UMLPLAIN/linux ]
 then
     cd $UMLPLAIN
-    lndir -silent $KERNPOOL .
+
+    lndirkerndirnogit $KERNPOOL .
 
     applypatches
 
@@ -214,7 +217,7 @@ done
 if $NEED_swan && [ ! -x $UMLSWAN/linux ]
 then
     cd $UMLSWAN
-    lndir -silent $KERNPOOL .
+    lndirkerndirnogit $KERNPOOL .
 
     applypatches
     
